# Integration tests specific to our eCards product
#

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.eCards.tests import base
from Products.CMFCore.utils import getToolByName
from Products.eCards.config import ALLTYPES, ALLSKINS

class TestProductUnInstallation(base.eCardTestCase):
    """ Ensure that our product plays nicely and uninstalls correctly.
    """
    def afterSetUp(self):
        self.types      = self.portal.portal_types
        self.properties = self.portal.portal_properties
        self.factory    = self.portal.portal_factory
        self.qi         = self.portal.portal_quickinstaller
        self.workflow   = self.portal.portal_workflow
        self.skins      = self.portal.portal_skins
        self.css        = self.portal.portal_css
        
        self.eCardTypes  = ALLTYPES
        self.eCardSkins  = ALLSKINS
        
        # uninstall our product
        if self.qi.isProductInstalled('eCards'):
            self.qi.uninstallProducts(products=['eCards',])
    
    def testQIDeemsProductUninstalled(self):
        """Make sure the product is uninstalled in the eyes of the portal_quickinstaller"""
        self.failIf(self.qi.isProductInstalled('eCards'))

    def testAdapterTypeNotRegisteredOnUninstall(self):
        for t in self.eCardTypes:
            self.failIf(t in self.types.objectIds(),
                "Type %s is still registered with the types tool after uninstallation." % t)

    def testFactoryTypesRemovedOnUninstall(self):
        for t in self.eCardTypes:
            self.failIf(t in self.factory.getFactoryTypes(),
                "Type %s is still a factory type after uninstallation." % t)

    def testTypesNotSearchedRemovedOnUninstall(self):
        # XXX - revisit me
        pass

    def testTypesNotListedRemovedOnUninstall(self):
        metaTypesNotToList  = self.properties.navtree_properties.getProperty('metaTypesNotToList')
        for t in self.eCardTypes:
            self.failIf(t in metaTypesNotToList,
                "Type %s is still in the list of metaTypesNotToList after uninstallation" % t)
    
    def testSkinsRemovedOnUninstall(self):
        """Our all important skin layer(s) should be registered with the site."""
        for prodSkin in self.eCardSkins:
            self.failIf(prodSkin in self.skins.objectIds(),
                "The skin %s is still registered with the skins tool after uninstallation" % prodSkin)
    
    def testSkinLayersRemovedOnUninstall(self):
        """We need our product's skin directories to show up below custom as one of the called
           upon layers of our skin's properties
        """
        for selection, layers in self.skins.getSkinPaths():
            for specific_layer in self.eCardSkins:
                self.failIf(specific_layer in layers, "The %s layer \
                    still appears in the layers of Plone's %s skin after uninstall" % (specific_layer,selection))
    
    def testWorkflowToolNoLongerMaintainsWorkflowChainForRemovedType(self):
        for t in self.eCardTypes:
            self.failIf(self.workflow._chains_by_type.has_key(t), "The %s type \
                still appears to have a workflow chain with the workflow tool" % t)
    
    def testECardsNoLongerInTypesUsingViewInListing(self):
        """Make sure the eCard type shows up in the list of types needing view action in listings
        """
        # make sure we haven't lost the defaults
        self.failUnless('Image' in self.properties.site_properties.typesUseViewActionInListings)
        # and that our type appears in the list
        self.failIf('eCard' in self.properties.site_properties.typesUseViewActionInListings)
    
    def testResourceRegistriesNoLongerManagingECardResources(self):
        """We ensure that our css is no longer lying around upon uninstallation.
        """
        self.failIf('ecards.css' in self.css.getResourceIds())
    

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductUnInstallation))
    return suite



## The Project Name
PROJECTNAME = "eCards"

## Globals variable
GLOBALS = globals()

## Permissions
DEFAULT_ADD_CONTENT_PERMISSION = "Add portal content"

## The skins dir
SKINS_DIR = 'skins'

## Archetype content objects associated with product
ALLTYPES = ('eCard', 'eCardCollection',)

## Skin directories associated with product
ALLSKINS = ('ecards_images', 'ecards_templates', 'ecards_styles')

## Addition permissions added to the eCards namespace
# XXX There has to be something I'm missing here
from Products.CMFCore import permissions as CMFCorePermissions
SendECard = "eCards: Send eCard"
CMFCorePermissions.setDefaultRoles(SendECard, ['Manager','Anonymous']) 




# Integration tests specific to our eCards product
#

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.eCards.tests import base
from Products.CMFCore.utils import getToolByName

from Products.eCards.interfaces import IECardCollection, IECard, \
    IECardCollectionView, IECardPopupView
from zope.interface.verify import verifyObject, verifyClass

from Products.eCards.content.ecard import eCard
from Products.eCards.content.ecardcollection import eCardCollection
from Products.eCards.browser import eCardCollectionView, eCardPopupView

class TestProductInterfaces(base.eCardTestCase):
    """ Ensure that our eCard product classes and objects
        fullfill their contractual interfaces
    """
    def afterSetUp(self):
        pass
    
    def testContentInterfaces(self):
        """ Some basic boiler plate testing of Interfaces and objects"""
        # verify IECardCollection
        self.failUnless(IECardCollection.implementedBy(eCardCollection))
        self.failUnless(verifyClass(IECardCollection, eCardCollection))
        
        # verify IECard
        self.failUnless(IECard.implementedBy(eCard))
        self.failUnless(verifyClass(IECard, eCard))
    
    def testObjectInstances(self):
        # setup our content objects
        self.setupContainedECard()
        
        # verify we're an object of the expected class
        self.failUnless(isinstance(self.folder.collection, eCardCollection))
        self.failUnless(isinstance(self.folder.collection.ecard, eCard))
        
    def testContentObjectsVerify(self):
        # setup our content objects
        self.setupContainedECard()
        
        self.failUnless(verifyObject(IECardCollection, self.folder.collection))
        self.failUnless(verifyObject(IECard, self.folder.collection.ecard))
    
    def testBrowserViewInterfaces(self):
        # verify IECardCollectionView
        self.failUnless(IECardCollectionView.implementedBy(eCardCollectionView))
        self.failUnless(verifyClass(IECardCollectionView, eCardCollectionView))
        
        # verify IECardPopupView
        self.failUnless(IECardPopupView.implementedBy(eCardPopupView))
        self.failUnless(verifyClass(IECardPopupView, eCardPopupView))
    
    def testBrowserViewsVerify(self):
        # just need a collection here
        self.setupCollection()
        self.setupContainedECard()
        
        # get our views
        collection_view = self.folder.collection.restrictedTraverse('ecardcollection_browserview')
        popup_view      = self.folder.collection.ecard.restrictedTraverse('ecardpopup_browserview')
        
        # verify we're an object of the expected class
        self.failUnless(isinstance(collection_view, eCardCollectionView))
        self.failUnless(verifyObject(IECardCollectionView, collection_view))
        
        self.failUnless(isinstance(popup_view, eCardPopupView))
        self.failUnless(verifyObject(IECardPopupView, popup_view))
    

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductInterfaces))
    return suite
        


from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from Products.eCards.config import PROJECTNAME, GLOBALS, \
    DEFAULT_ADD_CONTENT_PERMISSION, SKINS_DIR

from zope.i18nmessageid import MessageFactory
MessageFactory = MessageFactory('eCards')

registerDirectory(SKINS_DIR + '/ecards_images', GLOBALS)
registerDirectory(SKINS_DIR + '/ecards_templates', GLOBALS)
registerDirectory(SKINS_DIR + '/ecards_styles', GLOBALS)

def initialize(context):
    
    import content
    
    ##########
    # Add our content types
    #
    # This approach borrowed from ATContentTypes
    #
    listOfTypes = listTypes(PROJECTNAME)
    
    content_types, constructors, ftis = process_types(
        listOfTypes,
        PROJECTNAME)
    
    allTypes = zip(content_types, constructors)
    for atype, constructor in allTypes:
        kind = "%s: %s" % (PROJECTNAME, atype.archetype_name)
        
        permission = DEFAULT_ADD_CONTENT_PERMISSION
        
        utils.ContentInit(
            kind,
            content_types      = (atype,),
            permission         = permission,
            extra_constructors = (constructor,),
            fti                = ftis,
            ).initialize(context)



from zope.interface import Interface

class IECard(Interface):
    """eCard interface
    """
    def thumbtag():
        """Generate image tag using the api of the ImageField
        """
    

class IECardCollection(Interface):
    """eCard Collection interface
    """
    

class IECardCollectionView(Interface):
    """eCard Collection browser view
    """
    def getCardsForView():
        """Return a two dimensional array of images representing
           the number of images intended per row.
        """
    

class IECardPopupView(Interface):
    """eCard Collection browser view
    """
    def stripNewLines(str):
        """This method strips out linefeeds from
           any string in an effort to clamp down
           on potential spam sending vulnerabilities.
        """
    
    def sendECard():
        """Calls send on the MailHost object, protected
           by our custom Send eCard permission
        """
    

    


# Integration tests specific to our eCards product
#

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.eCards.tests import base
from Products.eCards.config import SendECard

class TestProductPermissions(base.eCardTestCase):
    """ Ensure that our eCard product classes and objects
        fullfill their contractual interfaces
    """
    def afterSetUp(self):
        # all of our tests need an eCard object
        self.setupCollection()
        self.setupContainedECard()
        self.ecard = self.folder.collection.ecard
        
        # get our view
        self.view = self.ecard.restrictedTraverse('ecardpopup_browserview')
        
        self.membership = self.portal.portal_membership
    
    def testCustomECardPermissionConfigured(self):
        """We setup a permission for the sending of eCards that can easily 
           be overridden in cases where this is wanted as a priveleged feature.
        """
        self.failUnless('eCards: Send eCard' in [r['name'] for r in self.portal.permissionsOfRole('Manager') if r['selected']])
        self.failUnless('eCards: Send eCard' in [r['name'] for r in self.portal.permissionsOfRole('Anonymous') if r['selected']])
    
    def testAnonHasAccessToPopupView(self):
        """Our view is protected by the eCards.SendECard permission
           We ensure that anonymous can access that view by default
        """
        self.logout()
        self.failUnless(self.portal.portal_membership.isAnonymousUser())
        self.failUnless(self.membership.checkPermission(SendECard, self.ecard))
        
    def testViewProtectedUponRoleModification(self):
        """Our view is protected by the eCards.SendECard permission, which
           by default should be allowable by all.
           
           However we want people to be able make this "anonymous"
           protected activity.  The following test demonstrates that.
        """
        self.portal.manage_permission(SendECard, ('Manager',), acquire=0)
        self.failIf('eCards: Send eCard' in [r['name'] for r in self.portal.permissionsOfRole('Anonymous') if r['selected']])
        
        # test permissions as anon
        self.logout()
        self.failUnless(self.portal.portal_membership.isAnonymousUser())
        self.failIf(self.membership.checkPermission(SendECard, self.ecard))
    

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductPermissions))
    return suite



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

from zope.interface import implements
from AccessControl import ClassSecurityInfo

from Products.CMFCore import permissions
from Products.eCards.config import PROJECTNAME
from Products.eCards.interfaces import IECardCollection

from Products.ATContentTypes.content.folder import ATFolderSchema, ATFolder
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

# XXX needed?
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixinSchema
from Products.ATContentTypes.interfaces import IATFolder

schema = ATFolderSchema.copy()

eCardCollectionSchema = schema + Schema((
    TextField('text',
        required = False,
        searchable = True,
        primary = True,
        storage = AnnotationStorage(migrate=True),
        validators = ('isTidyHtmlWithCleanup',),
        #validators = ('isTidyHtml',),
        default_content_type = zconf.ATNewsItem.default_content_type,
        default_output_type = 'text/x-html-safe',
        allowable_content_types = zconf.ATNewsItem.allowed_content_types,
        widget = RichWidget(
            description = "",
            description_msgid = "help_body_text",
            label = "Body Text",
            label_msgid = "label_body_text",
            rows = 25,
            i18n_domain = "plone",
            allow_file_upload = zconf.ATDocument.allow_document_upload)
        ),
    IntegerField('eCardsPerRow',
              languageIndependent=1,
              default=5,
              validators=('isInt',),
              widget=IntegerWidget(label='Number of eCard thumbnails per row',
                                label_msgid="label_ecardsPerRow_description",
                                description="""The default eCard Collection template outputs eCards \
                                    as thumbnails with the dimension of 128px by 128px in rows.  Depending \
                                    upon the width of the site's graphic design and the desired thumbnail size \
                                    , more or fewer per row will look appropriate.  Set the number of eCards per \
                                    row your graphic design can accommodate.""",
                                description_msgid="help_ecardsPerRow_description",
                                i18n_domain = "plone",)),
    StringField('thumbSize',
              languageIndependent=1,
              default='thumb',
              vocabulary=DisplayList((('original','Original size'),('mini','Mini (200px x 200px)'),\
                    ('thumb','Thumb (128px x 128px)'),('tile','Tile (64px x 64px)'),)),
              widget=SelectionWidget(label='Thumbnail Size (in pixels)',
                                format="select",
                                label_msgid="label_thumbSize_description",
                                description="""Override the default dimensions (128px x 128px) for your eCard thumbnails.  \
                                If you choose original, you should plan on creating a custom thumbnail for each of your eCard images.""",
                                description_msgid="help_thumbSize_description",
                                i18n_domain = "plone",)),
    TextField('emailSubject',
              languageIndependent=0,
              schemata='email',
              widget=StringWidget(label='Email Subject',
                                  label_msgid="label_email_subject",
                                  description="""This is the subject of the email sent to eCard \
                                               recipient.  This subject will be prepended with \
                                               sender's first name (i.e. a value like "has sent you an \
                                               eCard from example.com!" would result in "John has sent you an \
                                               eCard from example.com!").""",
                                  description_msgid="help_email_subject",
                                  i18n_domain = "plone",)),
    TextField('emailAppendedMessage',
              languageIndependent=0,
              schemata='email',
              default_content_type = zconf.ATNewsItem.default_content_type,
              default_output_type = 'text/x-html-safe',
              allowable_content_types = zconf.ATNewsItem.allowed_content_types,
              widget=RichWidget(label='Email Appended Message',
                                label_msgid="label_email_call_to_action",
                                description="""A default caption or message for all eCards within the collection \
                                    appearing below the eCard sender's custom message.  This is used \
                                    to encourage a relationship between the organization and the potential constituents \
                                    and/or customers receiving each eCard.""",
                                description_msgid="help_email_call_to_action",
                                i18n_domain = "plone",)),
    ),
)

finalizeATCTSchema(eCardCollectionSchema)

class eCardCollection(ATFolder):
    """eCard Collection is a folderish object that allows the addition of sortable eCards
    """
    implements(IECardCollection)
    
    schema = eCardCollectionSchema
    content_icon="ecardcollection_icon.gif"
    filter_content_types = 1
    allowed_content_types = ['eCard',]
    
    meta_type      = 'ECardCollection'
    archetype_name = 'eCard Collection'
    immediate_view = 'ecardcollection_view'
    default_view   = 'ecardcollection_view'
    _at_rename_after_creation = True
    
    security       = ClassSecurityInfo()
    
registerType(eCardCollection, PROJECTNAME)



import email 

from email.Header import Header
from email.MIMEText import MIMEText
from email.Utils import formatdate

from zope.interface import implements
from Acquisition import aq_parent
from Products import Five
from Products.CMFCore.utils import getToolByName

from Products.eCards.interfaces import IECardCollectionView, \
    IECardPopupView

class eCardCollectionView(Five.BrowserView):
    """See IECardCollectionView
    """
    implements(IECardCollectionView)
    
    def _tag_method(self, ecard):
        """Gets the preferred tag method 
           on the object dependent upon 
           the existing of an image in thumb field
        """
        if ecard.getThumb():
            return ecard.thumbtag
        else:
            return ecard.tag
    
    def _thumb_size(self, ecard):
        """Gets the preferred thumb tag
           method
        """
        collection = aq_parent(ecard)
        thumbName = collection.getThumbSize()
        
        if thumbName == 'original':
            return ''
            
        return "%s" % thumbName
    
    def getCardsForView(self):
        """See IECardCollectionView"""
        catalog = getToolByName(self.context, 'portal_catalog')
        params = {
            'portal_type':'eCard',
            'review_state':'published',
            'path':'/'.join(self.context.getPhysicalPath()),
            'sort_on':'getObjPositionInParent',
        }
        cards = catalog.searchResults(params)
        
        # need variables for 2-dim array setup
        numECardsPerRow = self.context.getECardsPerRow() or 5
        nestedCardList = []
        subCards = []
        
        for i in range(len(cards)):
            if len(subCards) == numECardsPerRow:
                # append to master list, create new subarray
                nestedCardList.append(subCards)
                subCards = []
            
            # naughty, but we need access to the object for PIL sized images
            card_obj = cards[i].getObject()
            tag = self._tag_method(card_obj)
            card_dict = {
                'title' : card_obj.Title(),
                'description' : card_obj.Description() or card_obj.Title(),
                'url' : card_obj.absolute_url(),
                'width' : card_obj.getWidth(),
            }
            
            # determine appropriate signature for tag method
            # dependant upon whether or not we're scaling
            if self._thumb_size(card_obj):
                card_dict['thumbnail_html'] = tag(scale=self._thumb_size(card_obj), css_class='ecardThumb')
            else:
                card_dict['thumbnail_html'] = tag(css_class='ecardThumb')
            
            subCards.append(card_dict)
            
            if i == len(cards) - 1:
                # we're the last item 
                # append empty items to our possibly partial list
                for i in range(numECardsPerRow - len(subCards)):
                    subCards.append('')
                
                # append the now expected length list to the master list
                nestedCardList.append(subCards)
        
        return nestedCardList

class eCardPopupView(Five.BrowserView):
    """See IECardPopupView
    """
    implements(IECardPopupView)
    
    def _make_html_email(self, **kw):
        """ Helper function relies heavily upon Python's 
            email Message module to construct our HTML mail.
        """
        msg = MIMEText(kw['message'], 'html',  _charset='UTF-8')
        
        msg['Subject'] = Header(kw['subject'], 'UTF-8')
        msg['From'] = kw['full_from_address']
        msg['To'] = kw['full_to_address']
        msg['Date'] = formatdate(localtime=True)
        msg["Message-ID"] = email.Utils.make_msgid()
        
        return msg
    
    def stripNewLines(self, str):
        """See IECardPopupView"""
        s = str.replace('\r', '')
        s = s.replace('\n', '')
        return s
    
    def sendECard(self, **kw):
        """See IECardPopupView"""
        msg = self._make_html_email(
            subject=kw['subject'],
            message=kw['message'],
            full_from_address=kw['full_from_address'],
            full_to_address=kw['full_to_address'])
        
        mailhost = getToolByName(self.context, 'MailHost')
        mailhost.send(msg.as_string(), kw['full_to_address'], kw['full_from_address'], kw['subject'])
    




from zope.interface import implements

# Import the base test case classes
from Testing import ZopeTestCase as ztc
from Products.CMFPlone.tests import PloneTestCase

# mock mailer
from Products.SecureMailHost.SecureMailHost import SecureMailHost as Base
from Products.MailHost.interfaces import IMailHost

# These must install cleanly, ZopeTestCase will take care of the others
ztc.installProduct('eCards')

# Set up the Plone site used for the test fixture. The PRODUCTS are the products
# to install in the Plone site (as opposed to the products defined above, which
# are all products available to Zope in the test fixture)
PRODUCTS = ['eCards']

PloneTestCase.setupPloneSite(products=PRODUCTS)

class eCardTestCase(PloneTestCase.PloneTestCase):
    """Base class for integration tests for the 'eCards' product. This may
    provide specific set-up and tear-down operations, or provide convenience
    methods.
    """
    
    def setupCollection(self):
        """We're needing to frequently setup an eCardCollection object
           We abstract this step out to our test base class, to save on the repetitive code.
        """
        self.folder.invokeFactory('eCardCollection', 'collection')
        
    def setupContainedECard(self):
        """We're needing to frequently setup a contained eCard object within a collection.
           We abstract this step out to our test base class, to save on the repetitive code.
        """
        if 'collection' not in self.folder.objectIds():
            # first setup a collection
            self.setupCollection()
            
        self.folder.collection.invokeFactory('eCard', 'ecard')
    
    def massECardProducer(self, numCards):
        """Some tests will depend upon a mass of
           eCards existing.  This method accepts
           an integer with a number of 
        """
        if 'collection' not in self.folder.objectIds():
            # first setup a collection
            self.setupCollection()
        
        # become master, so we can publish
        self.setRoles(['Manager',])
        
        if numCards:
            for cardidx in range(1, numCards):
                self.folder.collection.invokeFactory('eCard', "ecard%s" % str(cardidx))
                self.portal.portal_workflow.doActionFor(eval("self.folder.collection.ecard%s" % str(cardidx)), 'publish')
        
        # become normal test runner
        self.logout()
        self.login('test_user_1_')
    
    def determineRemainderOffset(self, numCards):
        """If we determine on our eCard Collection that we want 4 cards per row,
           but we've add 5 cards.  The two dimensional array will look like:
             [[1,2,3,4],[5,,,,]]
           
           Whereas if the user wants 4 cards per row and there are 4 cards, the array
           looks like:
           
             [[1,2,3,4],]
           
           This helper method helps us to determine whether we need to account for
           that remainder trailing subrow in our tests.
        """
        if numCards % self.folder.collection.getECardsPerRow() != 0:
            return 1
        
        return 0
        
class MailHostMock(Base):
    """
    mock up the send method so that emails do not actually get sent
    during unit tests we can use this to verify that the notification
    process is still working as expected
    """
    implements(IMailHost)
    
    def __init__(self, id):
        Base.__init__(self, id, smtp_notls=True)
        self.mail_text = ''
        self.n_mails = 0
        self.mto = ''
        self.mfrom = ''
        self.subject = ''
    def send(self, mail_text, full_to_address, full_from_address, subject):
        self.mail_text = mail_text
        self.mto = full_to_address
        self.mfrom = full_from_address
        self.subject = subject
        self.n_mails += 1



from Products.eCards.config import PROJECTNAME, ALLTYPES, ALLSKINS, SendECard
from Products.CMFCore.utils import getToolByName

from StringIO import StringIO

def uninstall(self):
    out = StringIO()
    
    portal_factory = getToolByName(self,'portal_factory')
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    types_tool = getToolByName(self, 'portal_types')

    # remove eCards as a factory types
    factory_types = portal_factory.getFactoryTypes().keys()
    for t in ALLTYPES:
        if t in factory_types:
            factory_types.remove(t)
    portal_factory.manage_setPortalFactoryTypes(listOfTypeIds=factory_types)
    print >> out, "Removed eCards types from portal_factory tool"
    
    # remove our types from the portal's list of types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f in typesNotListed:
            typesNotListed.remove(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Removed eCards types from metaTypesNotToList"
    
    # remove our types from the portal's list of types using view in action listings
    typesUseViewAction = list(siteProperties.getProperty('typesUseViewActionInListings'))
    for f in ('eCard',):
        if f in typesUseViewAction:
            typesUseViewAction.remove(f)
    siteProperties.manage_changeProperties(typesUseViewActionInListings = typesUseViewAction)
    print >> out, "Removed eCard from typesUseViewActionInListings"
    
    # Remove skin directory from skin selections
    skinstool = getToolByName(self, 'portal_skins')
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName)
        path = [i.strip() for i in  path.split(',')]
        for specific_layer in ALLSKINS:
            if specific_layer in path:
                path.remove(specific_layer)
        path = ','.join(path)
        skinstool.addSkinSelection(skinName, path)
    
    print >> out, "Removed skin layers from all skin selections"
    
    # clean-up mapped workflow chains for GS configured types
    workflow_tool = getToolByName(self, 'portal_workflow')
    del workflow_tool._chains_by_type['eCardCollection']
    print >> out, "Removed the associated eCardCollection type chains"
    
    print >> out, "\nSuccessfully uninstalled %s." % PROJECTNAME
    return out.getvalue()
    


try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

from zope.interface import implements

from Products.ATContentTypes.content.image import ATImage
from Products.ATContentTypes.content.image import ATImageSchema
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.base import ATCTFileContent
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.eCards.config import PROJECTNAME
from Products.eCards.interfaces import IECard

from Products.CMFCore import permissions
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo

from Products.Archetypes.Marshall import PrimaryFieldMarshaller

schema = ATImageSchema.copy()

schema = schema + Schema((
    TextField('credits',
              languageIndependent=0,
              widget=StringWidget(label='eCard Credits',
                                  label_msgid="label_e-card_credits",
                                  description='Give credit to the photographer of this e-card picture.',
                                  description_msgid="help_e-card_credits",
                                  i18n_domain = "plone")),
    ImageField('thumb',
              required=False,
              languageIndependent=True,
              storage = AnnotationStorage(migrate=True),
              swallowResizeExceptions = zconf.swallowImageResizeExceptions.enable,
              pil_quality = zconf.pil_config.quality,
              pil_resize_algo = zconf.pil_config.resize_algo,
              max_size = zconf.ATImage.max_image_dimension,
              sizes= {'mini'    : (200, 200),
                      'thumb'   : (128, 128),
                      'tile'    :  (64, 64),
                     },
              widget=ImageWidget(label='Image Thumbnail',
                                 label_msgid="label_e-card_thumbnail",                                 
                                 description='You may choose to upload your own thumbnail of any size to be \
                                    used within the eCard Collection view.  Using image resizing, a 200px, 128px, and \
                                    64px version will be created in addition to your original.  On your eCard Collection, \
                                    you can choose amongst several options for the thumbnail size.  If you do not upload a thumbnail,\
                                    the primary eCard image will be used and again, you can choose amongst several options for the \
                                    thumbnail size when editing your eCard Collection.',
                                 description_msgid="help_e-card_thumbnail XXX autogenerate",                                 
                                 i18n_domain = "plone")),
              ),
    )

finalizeATCTSchema(schema)

class eCard(ATImage):
    """This is the actual eCard image added to the eCard collection 
    """
    implements(IECard)
    
    schema = schema
    meta_type      = 'eCard'
    archetype_name = 'eCard'
    immediate_view = 'image_view'
    default_view   = 'image_view'
    global_allow   = 0
    
    security       = ClassSecurityInfo()
    
    security.declareProtected(permissions.View, 'tag')
    def thumbtag(self, **kwargs):
        """See IECard
        """
        return self.getField('thumb').tag(self, **kwargs)
    
    def __bobo_traverse__(self, REQUEST, name):
        """Transparent access to image scales
        """
        if name.startswith('image'):
            field = self.getField('image')
            image = None
            if name == 'image':
                image = field.getScale(self)
            else:
                scalename = name[len('image_'):]
                if scalename in field.getAvailableSizes(self):
                    image = field.getScale(self, scale=scalename)
            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image
        
        if name.startswith('thumb'):
            field = self.getField('thumb')
            thumb = None
            if name == 'thumb':
                thumb = field.getScale(self)
            else:
                scalename = name[len('thumb_'):]
                if scalename in field.getAvailableSizes(self):
                    thumb = field.getScale(self, scale=scalename)
            if thumb is not None and not isinstance(thumb, basestring):
                # thumb might be None or '' for empty thumbs
                return thumb

        return ATCTFileContent.__bobo_traverse__(self, REQUEST, name)
    
registerType(eCard, PROJECTNAME)


# Integration tests specific to our eCards product
#

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from base64 import decodestring
from Products.eCards.tests import base
from Products.CMFCore.utils import getToolByName
from Products.MailHost.interfaces import IMailHost

from Globals import package_home
PACKAGE_HOME = package_home(globals())

def loadImage(name, size=0):
    """Load image from testing directory
    """
    path = os.path.join(PACKAGE_HOME, 'input', name)
    fd = open(path, 'rb')
    data = fd.read()
    fd.close()
    return data

TEST_GIF = loadImage('test.gif')

class TestCollectionBrowserViews(base.eCardTestCase):
    """ Ensure that our eCardCollection's browser
        view
    """
    def afterSetUp(self):
        self.workflow = self.portal.portal_workflow
        
        # all of our tests need a collection object
        self.setupCollection()
        
        # get our view
        self.view = self.folder.collection.restrictedTraverse('ecardcollection_browserview')
    
    def testCollectionGetECardsForView(self):
        # setup our contained
        self.setupContainedECard()
        
        # we have subitems at this point, but our view method 
        # returns a false value as none of the sub eCards have been published
        self.failUnless(len(self.folder.collection.objectIds()))
        self.failIf(self.view.getCardsForView())
        
        # become manager (doesn't really matter how or by who this happens) publish 
        # our eCard and we should now be getting results per the catalog search criteria
        self.setRoles(['Manager',])
        self.workflow.doActionFor(self.folder.collection.ecard, 'publish')
        self.failUnless(self.view.getCardsForView())
        
        # become normal test runner
        self.logout()
        self.login('test_user_1_')
    
    def testCollectionViewECardsDictReturnsNeededData(self):
        # setup our contained
        self.setupContainedECard()
        
        # become manager (doesn't really matter how or by who this happens) publish 
        # our eCard and we should now be getting results per the catalog search criteria
        self.setRoles(['Manager',])
        self.workflow.doActionFor(self.folder.collection.ecard, 'publish')
        
        # become normal test runner
        self.logout()
        self.login('test_user_1_')
        
        # get first element of first subarray
        cardDict = self.view.getCardsForView()[0][0]
        
        for key in ('title','description','thumbnail_html','url','width',):
            self.failUnless(cardDict.has_key(key),"Key %s does not exist \
                in our card dict" % key)
    
    def testGetECardsPerRowReturnsEmptyListWithNoECards(self):
        # we really just want an empty list at first
        # the default eCardsPerRow value is 5, thus the tests are based on that
        self.assertEqual(5, self.folder.collection.getECardsPerRow())
        self.failIf(self.view.getCardsForView())
    
    def testGetECardsForViewTwoDimensionalArray(self):
        # first some setup, create n-1 eCards in our collection
        numCards = 7
        self.massECardProducer(numCards)
        
        # this should be numCards - 1 per range(1,numCards), see base.py
        cardsInCollection = len(self.folder.collection.objectIds())
        self.assertEqual(numCards - 1, cardsInCollection)
        
        # the default eCardsPerRow value is 5, thus the tests are based on that
        self.assertEqual(5, self.folder.collection.getECardsPerRow())        
        # we should have a two dimensional array with the following attributes 
        # (written in pseudocode):
        
        #   1) len(viewReturnedECards) == (cardsInCollection / eCardsPerRow)
        self.assertEqual(len(self.view.getCardsForView()), \
            ((cardsInCollection / self.folder.collection.getECardsPerRow()) + self.determineRemainderOffset(cardsInCollection))) 
        
        #   2) len(viewReturnedECards[-1]) == cardsInCollection % eCardsPerRow
        # our final, possibly unfilled element should have the full number items in its row
        self.assertEqual(len(self.view.getCardsForView()[-1]), self.folder.collection.getECardsPerRow())
        # we should however, only have the remainder of num cards divided by cards per row of 
        # meaningful data (i.e. not '') in the last element
        self.assertEqual(len([eCardDict for eCardDict in self.view.getCardsForView()[-1] if eCardDict != '']), \
           cardsInCollection % self.folder.collection.getECardsPerRow() or self.folder.collection.getECardsPerRow())
        
        # now lets update the number of items per row and recalculate
        self.folder.collection.setECardsPerRow(3)
        # we want a 2-dimensional array w/ possible final, unfilled last row (ala [item,'','',])
        self.assertEqual(len(self.view.getCardsForView()), \
            ((cardsInCollection / self.folder.collection.getECardsPerRow()) + self.determineRemainderOffset(cardsInCollection))) 
        # our final, possibly unfilled element should have the full number items in its row
        self.assertEqual(len(self.view.getCardsForView()[-1]), self.folder.collection.getECardsPerRow())
        # we should however, only have the remainder of num cards divided by cards per row of 
        # meaningful data (i.e. not '') in the last element
        self.assertEqual(len([eCardDict for eCardDict in self.view.getCardsForView()[-1] if eCardDict != '']), \
            cardsInCollection % self.folder.collection.getECardsPerRow() or self.folder.collection.getECardsPerRow())
        
        # now lets update the number of items per row to the number of cards contained and recalculate
        self.folder.collection.setECardsPerRow(6)
        self.assertEqual(len(self.view.getCardsForView()), \
            ((cardsInCollection / self.folder.collection.getECardsPerRow()) + self.determineRemainderOffset(cardsInCollection))) 
        self.assertEqual(len(self.view.getCardsForView()[-1]), \
            cardsInCollection % self.folder.collection.getECardsPerRow() or self.folder.collection.getECardsPerRow()) # XXX FIX ME --> [item,,,]
    
    def testAppropriateThumbSchemaImageReturnedByView(self):
        # setup our contained
        self.setupContainedECard()
        
        # become manager (doesn't really matter how or by who this happens) publish 
        # our eCard and we should now be getting results per the catalog search criteria
        self.setRoles(['Manager',])
        self.workflow.doActionFor(self.folder.collection.ecard, 'publish')
        
        # upload to image, test the html
        self.folder.collection.ecard.setImage(TEST_GIF, mimetype='image/gif', filename='test.gif')
        self.failUnless('image_thumb' in self.view.getCardsForView()[0][0]['thumbnail_html'])
        
        # upload to image, test the html
        self.folder.collection.ecard.setThumb(TEST_GIF, mimetype='image/gif', filename='testthumb.gif')
        self.failUnless('thumb_thumb' in self.view.getCardsForView()[0][0]['thumbnail_html'])
        
        # become normal test runner
        self.logout()
        self.login('test_user_1_')
    
    def testThumbSizeOverrideableOnCollectionLevel(self):
        # setup our contained
        self.setupContainedECard()

        # become manager (doesn't really matter how or by who this happens) to publish 
        # our eCard and we should now be getting results per the catalog search criteria
        self.setRoles(['Manager',])
        self.workflow.doActionFor(self.folder.collection.ecard, 'publish')

        # upload to image, test the html
        self.folder.collection.ecard.setImage(TEST_GIF, mimetype='image/gif', filename='test.gif')
        self.failUnless('image_thumb' in self.view.getCardsForView()[0][0]['thumbnail_html'])

        # now upload the thumb to ensure that our field namespace is correct
        self.folder.collection.ecard.setThumb(TEST_GIF, mimetype='image/gif', filename='testthumb.gif')
        self.failUnless('thumb_thumb' in self.view.getCardsForView()[0][0]['thumbnail_html'])
        
        # test thumb as mini
        self.folder.collection.setThumbSize('mini')
        self.failUnless('thumb_mini' in self.view.getCardsForView()[0][0]['thumbnail_html'])
        
        # test thumb as original
        self.folder.collection.setThumbSize('original')
        self.failUnless('thumb' in self.view.getCardsForView()[0][0]['thumbnail_html'])
    
    def testViewReturnsECardsWithinCollectionOnly(self):
        # setup our collection & contained
        self.setupContainedECard()
        
        # setup an alternate collection and ecard
        self.folder.invokeFactory('eCardCollection', 'mycollection')
        self.folder.mycollection.invokeFactory('eCard', 'myecard')
        
        # become manager (doesn't really matter how or by who this happens) publish 
        # our eCard and we should now be getting results per the catalog search criteria
        self.setRoles(['Manager',])
        self.workflow.doActionFor(self.folder.collection.ecard, 'publish')
        self.workflow.doActionFor(self.folder.mycollection.myecard, 'publish')
        
        # become normal test runner
        self.logout()
        self.login('test_user_1_')
        
        # make sure our view isn't returning ecards from all collections
        # we can't return more than exist
        self.failIf(len(self.folder.collection.objectIds()) < \
            len([eCardDict for eCardDict in self.view.getCardsForView()[0] if eCardDict != '']))
    
    def testViewReturnsECardsAsOrderedByPositionInParent(self):
        # setup several eCards
        for i in range(4):
            # create
            self.folder.collection.invokeFactory('eCard', 'ecard%s' % i)
            # workflow transition
            self.setRoles(['Manager',])
            self.workflow.doActionFor(self.folder.collection['ecard%s' % i], 'publish')
          
        # reorder, 2 moves to first, 1 moves to last
        self.folder.collection.moveObjectsByDelta(['ecard2'], -100)
        self.folder.collection.moveObjectsByDelta(['ecard1'], +100)
        
        for i in range(4):
            self.folder.collection['ecard%s' % i].reindexObject(idxs=['getObjPositionInParent',])
        
        # become normal test runner
        self.logout()
        self.login('test_user_1_')
        
        self.assertEqual([ecard_obj.absolute_url() for ecard_obj in self.folder.collection.objectValues()], 
                         [ecard['url'] for ecard in self.view.getCardsForView()[0] if ecard != ''])
        
        
        

class TestECardPopupBrowserViews(base.eCardTestCase):
    """ Ensure that our eCardCollection's browser
        view
    """
    def afterSetUp(self):
        
        # setup MailHostMock
        sm = self.portal.getSiteManager()
        sm.registerUtility(base.MailHostMock('MailHost'), IMailHost)
        self.mailhost = getToolByName(self.portal, 'MailHost')
        
        # all of our tests need an eCard object
        self.setupCollection()
        self.setupContainedECard()
        
        # get our view
        self.view = self.folder.collection.ecard.restrictedTraverse('ecardpopup_browserview')
    
    def testPopupViewSendsEmail(self):
        # mock form vars
        mail_to = 'jane@example.com'
        mail_from = 'john@example.com'
        
        options = {
            'subject': "Jane has sent you an eCard!",
            'friend_first_name': "Jane",
            'comment': "You'll love this picture!",
            'sender_first_name': "John",
            'credits': "Photo courtesy of Plone Foundation",
            # XXX change me
            'emailAppendedMessage': "Make a contribution at Plone.org",
            'image_url': "http://nohost/ecardimage",
        }
        
        # trigger the email
        self.view.sendECard(
            message = self.portal.portal_skins.ecards_templates.email_template(**options),
            full_to_address = mail_to,
            full_from_address = mail_from,
            subject = options['subject'],
        )
        
        # assert that a message was sent and it had some realistic data
        self.assertEqual(self.mailhost.n_mails, 1)
        self.assertEqual(self.mailhost.mto, mail_to)
        self.assertEqual(self.mailhost.mfrom, mail_from)
        decoded_mail_text = decodestring(self.mailhost.mail_text)
        for v in options.values():
            self.failUnless(v in decoded_mail_text, "The value %s was not found in the mail text")
    
    def testPopupViewStripNewLines(self):
        for s in ('Message', '\nMessage', '\rMessage', '\n\rMessage\n\r'):
            self.assertEqual('Message', self.view.stripNewLines(s))


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCollectionBrowserViews))
    suite.addTest(makeSuite(TestECardPopupBrowserViews))
    return suite
    


# Integration tests specific to our eCards product
#

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.eCards.tests import base
from Products.CMFCore.utils import getToolByName

from Globals import package_home
PACKAGE_HOME = package_home(globals())

def loadImage(name, size=0):
    """Load image from testing directory
    """
    path = os.path.join(PACKAGE_HOME, 'input', name)
    fd = open(path, 'rb')
    data = fd.read()
    fd.close()
    return data

TEST_GIF = loadImage('test.gif')


class TestContentEditing(base.eCardTestCase):
    """ Ensure that our eCard product installs and 
        correctly configures our portal.
    """
    def afterSetUp(self):
        self.properties = self.portal.portal_properties
        self.types      = self.portal.portal_types
    
    def testECardCollectionAddable(self):
        self.folder.invokeFactory('eCardCollection', 'collection')
        self.failUnless('collection' in self.folder.objectIds())
    
    def testSetGetOnECardCollection(self):
        # add the collection
        self.folder.invokeFactory('eCardCollection', 'collection')
        
        # set all fields
        # XXX -- finish me!!
        self.folder.collection.setTitle('My eCard Collection')
        self.folder.collection.setDescription('My description')
        self.folder.collection.setText('<p>Some Body text</p>')
        self.folder.collection.setEmailSubject('Your friend sent you an eCard')
        self.folder.collection.setEmailAppendedMessage('<p>Some more information about the organization that made this eCard possible</p>')
        self.folder.collection.setECardsPerRow(5)
        self.folder.collection.setThumbSize('tile')
        
        # test the field values
        self.assertEqual(self.folder.collection.Title(), 'My eCard Collection')
        self.assertEqual(self.folder.collection.Description(), 'My description')
        self.assertEqual(self.folder.collection.getText(), '<p>Some Body text</p>')
        self.assertEqual(self.folder.collection.getEmailSubject(), 'Your friend sent you an eCard')
        self.assertEqual(self.folder.collection.getEmailAppendedMessage(), '<p>Some more information about the organization that made this eCard possible</p>')
        self.assertEqual(self.folder.collection.getECardsPerRow(),5)
        self.assertEqual(self.folder.collection.getThumbSize(),'tile')
    
    def testECardCollectionCardsPerRowIntegerOnly(self):
        """Because code in our eCardCollectionView browser based
           view depends on receiving an integer for mathematical 
           operations, we make sure in our test that this doesn't change.
        """
        self.folder.invokeFactory('eCardCollection', 'collection')
        self.assertRaises(ValueError,self.folder.collection.setECardsPerRow,'Some String')
    
    def testECardCollectionSchemata(self):
        """Make sure that we have our needed schemata for configuration of 
           email settings.
        """
        # add the collection
        self.folder.invokeFactory('eCardCollection', 'collection')
        self.failUnless('email' in self.folder.collection.Schema().getSchemataNames())
    
    def testECardAddableToCollection(self):
        self.folder.invokeFactory('eCardCollection', 'collection')
        self.folder.collection.invokeFactory('eCard', 'ecard')
        self.failUnless('ecard' in self.folder.collection.objectIds())
    
    def testECardNotGloballyAddable(self):
        self.assertRaises(ValueError, self.folder.invokeFactory, 'eCard', 'ecard')
    
    def testSetGetOnECard(self):
        """Make sure we can add the actual eCard successfully"""
        # add the collection and contained eCard
        self.folder.invokeFactory('eCardCollection', 'collection')
        self.folder.collection.invokeFactory('eCard', 'ecard')
        
        # set all fields
        # XXX -- finish me!!
        self.folder.collection.ecard.setCredits('Some great photographer')
        self.folder.collection.ecard.setImage(TEST_GIF, mimetype='image/gif', filename='test.gif')
        self.folder.collection.ecard.setThumb(TEST_GIF, mimetype='image/gif', filename='test.gif')
        
        # test the field values
        self.assertEqual(self.folder.collection.ecard.getCredits(), 'Some great photographer')
        self.failUnlessEqual(self.folder.collection.ecard.getImage().data, TEST_GIF)
        self.failUnlessEqual(self.folder.collection.ecard.getThumb().data, TEST_GIF)
    
    def testThumbTagOnECard(self):
        # add the collection and contained eCard
        self.setupContainedECard()
        self.folder.collection.ecard.setThumb(TEST_GIF, mimetype='image/gif', filename='test.gif')
        self.failUnless('128' in self.folder.collection.ecard.thumbtag(scale='thumb'))
        self.failUnless('thumb_thumb' in self.folder.collection.ecard.thumbtag(scale='thumb'))
    
    def testCollectionEmailAppendedMessageDefaultsToCorrectFormat(self):
        """Ensure that the default format for our eCard Collection's
           HTML-rich email body is actually HTML.
        """
        self.setupCollection()
        self.assertEqual('text/html', self.folder.collection.Schema().get('emailAppendedMessage').default_content_type)
    
    def testCollectionCanBeOrderedInParent(self):
        """See: http://plone.org/products/ecards/issues/10
           Thanks Weblion: https://weblion.psu.edu/trac/weblion/changeset/2608
        """
        # setup several collections
        self.folder.invokeFactory('eCardCollection', 'collection1')
        self.folder.invokeFactory('eCardCollection', 'collection2')
        self.folder.invokeFactory('eCardCollection', 'collection3')
        
        # prove that we can reorder
        self.folder.moveObjectsByDelta(['collection3'], -100)
        self.failUnless(self.folder.getObjectPosition('collection3') == 0,
            "eCardCollection subobject 'collection3' should be at position 0.")
        
        

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestContentEditing))
    return suite



# Integration tests specific to our eCards product
#

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.eCards.tests import base
from Products.CMFCore.utils import getToolByName
from Products.eCards.config import ALLTYPES, ALLSKINS

class TestProductInstallation(base.eCardTestCase):
    """ Ensure that our eCard product installs and 
        correctly configures our portal.
    """
    def afterSetUp(self):
        self.properties = self.portal.portal_properties
        self.factory    = self.portal.portal_factory
        self.types      = self.portal.portal_types
        self.skins      = self.portal.portal_skins
        self.workflow   = self.portal.portal_workflow
        self.css        = self.portal.portal_css
        
        self.eCardTypes  = ALLTYPES
        self.eCardSkins  = ALLSKINS
    
    def testCoreProductsInstalled(self):
        CORE_PRODUCTS = ['eCards',]
        
        for prod in CORE_PRODUCTS:
            self.failUnless(self.portal.portal_quickinstaller.isProductInstalled(prod),
                "Core product %s is not already installed" % prod)
    
    def testPortalFactorySetup(self):
        for t in self.eCardTypes:
            self.failUnless(t in self.factory.getFactoryTypes(),
                "Type %s is not a factory types" % t)
    
    def testStockListedTypesNotPurged(self):
        metaTypesNotToList  = self.properties.navtree_properties.getProperty('metaTypesNotToList')
        self.failUnless(len(metaTypesNotToList) > len(self.eCardTypes))
    
    def testTypesNotListed(self):
        metaTypesNotToList  = self.properties.navtree_properties.getProperty('metaTypesNotToList')
        for t in self.eCardTypes:
            self.failUnless(t in metaTypesNotToList,
                "Type %s will show up in the nav and shouldn't" % t)
    
    def testCorePloneLayersStillPresent(self):
        """As was known to happen with legacy versions of GS working with the skins tool,
           it was easy to inadvertently remove need layers for the selected skin.  Here 
           we make sure this hasn't happened.
        """
        core_layers = ('plone_templates','plone_content',)
        
        for selection, layers in self.skins.getSkinPaths():
            for specific_layer in core_layers:
                self.failUnless(specific_layer in layers, "The %s layer \
                    does not appear in the layers of Plone's available skins" % specific_layer)
    
    def testECardTypesFTI(self):
        """We keep the eCardFTIMapping up to date as a
           static list of what the very important FTI settings
           are for our subclassed ATCT types and make sure they
           match the settings in the ZMI's portal_types tool.
        """
        eCardFTIMapping = [
            {"eCard":{
                    "title":"eCard",
                    "content_meta_type":"eCard",
                    "product":"eCards",
                    "global_allow":False,
                    "filter_content_types":False,
                    "content_icon":'ecard_icon.gif',
                    "factory":'addeCard',
                },},
            
            {"eCardCollection":{
                    "title":"eCard Collection",
                    "content_meta_type":"ECardCollection",
                    "product":"eCards",
                    "global_allow":True,
                    "filter_content_types":True,
                    "allowed_content_types":('eCard',),
                    "content_icon":'ecardcollection_icon.gif',
                    "factory":'addeCardCollection',
                },},
            
        ]
        
        for fti_type in eCardFTIMapping:
            for k,v in fti_type.items():
                type_obj = getattr(self.types,k)
                for type_attr, type_attr_value in v.items():
                    self.assertEquals(eval("type_obj.%s" % type_attr), type_attr_value,
                        "For type %s, the FTI value should be %s for the %s field \
                        types tool, but it's %s" % (k, type_attr_value, type_attr, eval("type_obj.%s" % type_attr)))
    
    def testBrowseToObjectViaViewAlias(self):
        """Test to prove fix to bug where upon adding an eCard to a collection
           one was taken directly to the index_html method, which on an image
           is the image within the web browser, which thereby takes the content
           edit away from the Plone UI, which is very baffling.
        """
        ecard_fti = self.types.eCard
        self.failUnless("/view" in \
            ecard_fti.getActionObject('object/view').getActionExpression())
    
    def testFSDVInSkinsTool(self):
        """Test the the needed file system directory views have
           been registered with the skins tool.
        """
        for prodSkin in self.eCardSkins:
            self.failUnless(prodSkin in self.skins.objectIds(),
                "The skin %s was not registered with the skins tool" % prodSkin)
    
    def testFSDVContainSkinResourcesTemplates(self):
        """This is a bit of overkill, but since eCards is being targetted
           past pre-customerize Plone releases, we want various aspects to be
           customizable via the ZMI.  We make sure those items, roughly, exist
           in their appropriate skin layers. This also provides some coverage
           of a missing registerDirectory call w/in __init__ for the package.
        """
        self.failUnless('ecardcollection_icon.gif' in self.skins.ecards_images.objectIds())
        self.failUnless('ecardcollection_view' in self.skins.ecards_templates.objectIds())
        self.failUnless('ecards.css' in self.skins.ecards_styles.objectIds())
    
    def testSkinShowsUpInToolsLayers(self):
        """We need our product's skin directories to show up below custom as one of the called
           upon layers of our skin's properties
        """
        for selection, layers in self.skins.getSkinPaths():
            for specific_layer in self.eCardSkins:
                self.failUnless(specific_layer in layers, "The %s layer \
                    does not appear in the layers of Plone's %s skin" % (specific_layer,selection))
    
    def testCorePloneLayersStillPresent(self):
        """As was known to happen with legacy versions of GS working with the skins tool,
           it was easy to inadvertently remove need layers for the selected skin.  Here 
           we make sure this hasn't happened.
        """
        core_layers = ('plone_styles','plone_templates',)
        
        for selection, layers in self.skins.getSkinPaths():
            for specific_layer in core_layers:
                self.failUnless(specific_layer in layers, "The %s layer \
                    does not appear in the layers of Plone's available skins" % specific_layer)
    
    def testChainsForECardTypes(self):
        """Make sure that the workflow chains for our
           eCard types are resonable and reflect the expected
           Plone defaults
        """
        self.assertEqual('plone_workflow',self.workflow.getChainForPortalType('eCard')[0])
        self.assertEqual('folder_workflow',self.workflow.getChainForPortalType('eCardCollection')[0])
    
    def testECardsInTypesUsingViewInListing(self):
        """Make sure the eCard type shows up in the list of types needing view action in listings
        """
        # make sure we haven't lost the defaults
        self.failUnless('Image' in self.properties.site_properties.typesUseViewActionInListings)
        # and that our type appears in the list
        self.failUnless('eCard' in self.properties.site_properties.typesUseViewActionInListings)
    
    def testResourceRegistriesManagingCSSResources(self):
        """We register a css stylesheet on installation.  
           Here we ensure correct setup.
        """
        self.failUnless('ecards.css' in self.css.getResourceIds())
    

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductInstallation))
    return suite
